'
'  ras_wrap.bas
'
'  Ras Services wrapper functions.
'
'


'- Conditionally, this can be compiled as a DLL,
'  set the constant below to be non-zero if you
'  wish to have a DLL.
'
%COMPILE_AS_DLL = 0
$IF %COMPILE_AS_DLL

   $COMPILE DLL "ras32.dll"
   '- This is the only API call required, so I don't bother including
   '  win32api.inc if we're doing a DLL
   '
   %False =  0
   %True  = -1
   DECLARE SUB apiSleep LIB "KERNEL32.DLL" ALIAS "Sleep" (BYVAL dwMilliseconds AS LONG)

$ENDIF

$INCLUDE "RAS32.INC"


'
'  load_ras_entries                 NOT EXPORTED
'
'  Fills the given array with the entries in the ras phone book.
'  The array rEntry is 1-based.
'  Returns the number of entries in the array.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function load_ras_entries(rEntry() As RASENTRYNAME) As Long 

   Dim iRet As Long
   Dim iBufSize As Long
   Dim iNumEntries As Long

   ReDim rEntry(1 To 1) As RASENTRYNAME
   iBufSize = SizeOf(rEntry(1))
   rEntry(1).dwSize = SizeOf(rEntry(1))
   iRet = RasEnumEntries(ByVal 0, ByVal 0, rEntry(1), iBufSize, iNumEntries)

   '- If we have more than one entry, we'll need
   '  to increase the size of the array.
   '
   If iRet = %ERROR_BUFFER_TOO_SMALL Then
      ReDim rEntry(1 To iNumEntries) As RASENTRYNAME
      iBufSize = SizeOf(rEntry(1)) * iNumEntries
      rEntry(1).dwSize = SizeOf(rEntry(1))
      iRet = RasEnumEntries(ByVal 0, ByVal 0, rEntry(1), iBufSize, iNumEntries)
   End If

   If iRet <> 0 Then
      Function = 0
   Else
      Function = iNumEntries
   End If

End Function

'
'  hangup_and_wait                  NOT EXPORTED
'
'  This routine hangs up a ras connection. Because it takes the program time
'  to disconnect, we need to wait until the ras handle is invalidated.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub hangup_and_wait(ByVal hRas As Long)

   Dim iRet As Long
   Dim rStatus As RASCONNSTATUS

   RasHangUp hRas

   Do
      rStatus.dwSize = SizeOf(rStatus)
      iRet = RasGetConnectStatus(hRas, rStatus)
      apiSleep 5
   Loop Until iRet <> 0

End Sub

'
'  collect_active                   NOT EXPORTED
'  
'  Takes an array of structures to fill with information
'  about the active connections.
'  Returns the number of connections found and zero on error.
'
Function collect_active(rConn() as RASCONN) as LONG

   Dim iBufSize as Long
   Dim iNumConn as Long
   Dim iRet as Long
   Dim rConn(1 To 100) As RASCONN      'assuming never more than 100
                                       'active connections (safe for now!)
   
   iBufSize = SizeOf(rConn(1)) * ubound(rConn)
   rConn(1).dwSize = SizeOf(rConn(1))
   iRet = RasEnumConnections(rConn(1), iBufSize, iNumConn)
   If iRet = 0 Then
      FUNCTION =iNumConn
   Else
      FUNCTION = %False
   End If

End Function


'
'  rsAvailableEntries
'
'  Returns a count of the available entries
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsAvailableEntries Alias "rsAvailableEntries" EXPORT As Long

   Dim rEntry(1) As RASENTRYNAME

   Function = load_ras_entries(rEntry())

End Function

'
'  rsGetEntryName
'
'  Returns a RAS entry by name based in it's number
'  as returned by rsAvailableEntries
'
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsGetEntryName Alias "rsGetEntryName" (ByVal iEntry As Long) EXPORT As String

   Dim iCount As Long
   Dim rEntry(1) As RASENTRYNAME

   iCount = load_ras_entries(rEntry())
   If (iEntry < 1) Or (iEntry > iCount) Then
      Function = ""
   Else
      Function =  Trim$(rEntry(iEntry).szEntryName)
   End If

End Function

'
'  rsDialEntry
'
'  Dials a RAS Connection
'     Returns  non-zero RAS Handle On success
'              %False On Failure
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsDialEntry Alias "rsDialEntry" (ByVal iEntry As Long) EXPORT As Long

   Dim hRas As Long
   Dim iRet As Long
   Dim iCount As Long
   Dim iPassword As Long
   Dim rEntry(1) As RASENTRYNAME
   Dim rDial As RASDIALPARAMS
   Dim rDialEx As RASDIALEXTENSIONS
   Dim rStatus As RASCONNSTATUS

   '- Find the entry we want
   iCount = load_ras_entries(rEntry())
   If (iCount < 1) Or (iEntry < 1) Or (iEntry > iCount) Then
      Function = %False
   Else

      '- Get it's dialing parameters
      rDial.dwSize = SizeOf(rDial)
      rDial.szEntryName = Trim$(rEntry(iEntry).szEntryName)
      iRet = RasGetEntryDialParams(ByVal 0, rDial, iPassword)
      If iRet <> 0 Then
         Function = %False
      Else

         '- Make the connection
         rDial.dwSize = SizeOf(rDial)
         rDialEx.dwSize = SizeOf(rDialEx)
         iRet = RasDial ( rDialEx, ByVal 0, rDial, 0, 0, hRas )

         '- Check for an error
         If iRet <> 0 Then
            hangup_and_wait hRas
            Function = %False
         Else

            '- Get the connections status
            rStatus.dwSize = SizeOf(rStatus)
            If RasGetConnectStatus(hRas, rStatus) <> 0 Then
               hangup_and_wait hRas
               Function = %False
            Else
               If rStatus.rasconnstate = %RASCS_CONNECTED Then
                  Function = hRas
               Else
                  hangup_and_wait hRas
                  Function = %False
               End If
            End If
         End If
      End If
   End If

End Function

'  
'  rsActiveConnections
'
'  Returns a count of active connections
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsActiveConnections Alias "rsActiveConnections" EXPORT as Long

   Dim rConn(1 to 100) as RASCONN
   
   FUNCTION = collect_active(rConn())
   
End Function

'
'  rsGetConnectionName
'
'  Returns the entry name of an active connection based on it's "number"
'  as returned by rsActiveConnections
'
Function rsGetConnectionName Alias "rsGetConnectionName" ( ByVal iConnection As Long ) EXPORT As String
   
   Dim iNumConn as Long
   Dim rConn(1 To 100) As RASCONN      'assuming never more than 100
                                       'active connections (safe for now!)
                                          
   iNumConn = collect_active(rConn())
   if (iConnection < 1) or (iConnection > iNumConn) then
      Function = ""
   else
      Function = Trim$(rConn(1).szEntryName)
   end if
   
End Function

'
'  rsGetConnectionHandle
'
'  Returns the hRas handle to the active connection based on it's "number"
'  as specified by rsActiveConnections. Returns Zero on error.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsGetConnectionHandle Alias "rsGetConnectionHandle" ( ByVal iConnection As Long ) EXPORT As Long
   
   Dim iNumConn as Long
   Dim rConn(1 To 100) As RASCONN      'assuming never more than 100
                                       'active connections (safe for now!)
                                          
   iNumConn = collect_active(rConn())
   if (iConnection < 1) or (iConnection > iNumConn) then
      Function = 0
   else
      Function = rConn(1).hRasConn
   end if
   
End Function

'
'  rsHangupConnection
'
'  Hangs up a connection given it's handle and doesn't return
'  until the connection is disconnected.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function rsHangUpConnection Alias "rsHangUpConnection" ( ByVal hRas As Long ) EXPORT As Long

   hangup_and_wait hRas

End Function

                                                                             
